package crafttweaker.mc1120.item;

import crafttweaker.api.data.IData;
import crafttweaker.api.entity.*;
import crafttweaker.api.item.*;
import crafttweaker.api.potions.IPotion;
import crafttweaker.mc1120.brackets.BracketHandlerItem;
import crafttweaker.mc1120.commands.Commands;
import crafttweaker.mc1120.data.NBTConverter;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.init.Items;
import net.minecraft.item.*;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.potion.*;
import net.minecraft.util.*;
import stanhebben.zenscript.annotations.Optional;

import java.util.*;
import java.util.regex.*;

/**
 * @author BloodWorkXGaming
 */
public class MCItemUtils implements IItemUtils{
    
    private static ArrayList<ItemStack> ITEMLIST;
    
    public static void createItemList() {
        ArrayList<ItemStack> itemList = new ArrayList<>();
        
        List<Item> items = new ArrayList<>(BracketHandlerItem.getItemNames().values());
        items.sort(Commands.ITEM_COMPARATOR);
        
        for(Item item : items) {
            if(item != null) {
                // gets list of subitems
                NonNullList<ItemStack> list = NonNullList.func_191196_a();
                item.func_150895_a(CreativeTabs.field_78027_g, list);
                
                if(list.size() > 0) {
                    itemList.addAll(list);
                } else {
                    ItemStack stack = new ItemStack(item, 1, 0);
                    itemList.add(stack);
                }
            }
        }
        
        ITEMLIST = itemList;
    }
    
    
    /**
     *
     * @param params has to be in this syntax:
     *               [<effect1>, strength, time], [<effect2>, strength, time], ...
     * @return returns the {@link IItemStack} of the potion
     */
    public IItemStack createPotion(Object[]... params){
        ItemStack item = new ItemStack(Items.field_151068_bn,1, 0);
    
        List<PotionEffect> potionEffects = new ArrayList<>();
        
        for(Object[] param: params) {
            IPotion iPotion;
            int amplifier;
            int duration;
            
            if (param.length == 3){
                if (param[0] instanceof IPotion){
                    iPotion = (IPotion) param[0];
                }else{
                    continue;
                }
                
                if (param[1] instanceof Integer){
                    amplifier = (int) param[1];
                }else{
                    continue;
                }
                
                if (param[2] instanceof Integer){
                    duration = (int) param[2];
                }else{
                    continue;
                }
                PotionEffect effect = new PotionEffect(((Potion) iPotion.getInternal()), duration, amplifier);
                potionEffects.add(effect);
            
            }
        }
        
        // potionEffects.forEach(it -> System.out.println(it.toString()));
        PotionUtils.func_185184_a(item, potionEffects);
        
        return new MCItemStack(item);
    }

    
    public IItemStack[] getItemsByRegexRegistryName(String regex){
        ArrayList<ItemStack> matchedItems = new ArrayList<>();
        HashSet<String> alreadyChecked = new HashSet<>();
        Pattern p = Pattern.compile(regex);
        
        for(ItemStack stack : ITEMLIST) {
            String currentRegName = stack.func_77973_b().getRegistryName().toString() + ":" + stack.func_77960_j();
            // prevent duplicate checks on items with different NBT(which I ignore here)
            if (alreadyChecked.contains(currentRegName)){
                continue;
            }
            alreadyChecked.add(currentRegName);
            
            Matcher m = p.matcher(currentRegName);
            if (m.matches()){
                // clears away meta if it has
                if (stack.func_77942_o()){
                    matchedItems.add(new ItemStack(stack.func_77973_b(), 1, stack.func_77960_j()));
                }else {
                    matchedItems.add(stack);
                }
                // System.out.println("Matched regName: " + currentRegName);
            }
        }
    
        IItemStack[] iItemStacks = new IItemStack[matchedItems.size()];
        for(int i = 0; i < matchedItems.size(); i++) {
            iItemStacks[i] = new MCItemStack(matchedItems.get(i));
        }
    
        return iItemStacks;
    }
    
    
    public IItemStack[] getItemsByRegexUnlocalizedName(String regex){
        ArrayList<ItemStack> matchedItems = new ArrayList<>();
        Pattern p = Pattern.compile(regex);
    
        for(ItemStack stack : ITEMLIST) {
            String currentUnlocalizedName = stack.func_77977_a();
        
            Matcher m = p.matcher(currentUnlocalizedName);
            if (m.matches()){
                matchedItems.add(stack);
                // String currentRegName = stack.getItem().getRegistryName().toString() + ":" + stack.getMetadata();
                // System.out.println("Matched unlocalized: " + currentUnlocalizedName + " [" + currentRegName + "]");
            }
        }
        
        IItemStack[] iItemStacks = new IItemStack[matchedItems.size()];
        for(int i = 0; i < matchedItems.size(); i++) {
            iItemStacks[i] = new MCItemStack(matchedItems.get(i));
        }
        
        return iItemStacks;
    }
    

    public IItemStack createSpawnEgg(IEntityDefinition entity, @Optional IData customNBT){
        ItemStack item = new ItemStack(Items.field_151063_bx,1, 0);
        NBTTagCompound baseTag;
        if (customNBT != null){
            baseTag = (NBTTagCompound) NBTConverter.from(customNBT);
        }else {
            baseTag = new NBTTagCompound();
        }
    
        NBTTagCompound entityTag;
        if (baseTag.func_74764_b("EntityTag")){
            entityTag = baseTag.func_74775_l("EntityTag");
        }else {
            entityTag = new NBTTagCompound();
        }
        
        entityTag.func_74778_a("id", entity.getId());
        baseTag.func_74782_a("EntityTag", entityTag);
        item.func_77982_d(baseTag);
        
        return new MCItemStack(item);
    }
    
    
}
